<!-- Generated by cargo-onedoc v0.2.2. DO NOT EDIT. -->

# fmutex

[![Crates.io Version](https://badgers.space/crates/version/fmutex)](https://crates.io/crates/fmutex)
[![Docs.rs Latest](https://badgers.space/badge/docs.rs/latest/blue)](https://docs.rs/fmutex)
[![Build Status](https://badgers.space/github/checks/rossmacarthur/fmutex?label=build)](https://github.com/rossmacarthur/fmutex/actions/workflows/build.yaml)

Mutual exclusion across processes on a file descriptor or path.

- On Unix-like systems this is implemented use
  [`flock(2)`](https://man7.org/linux/man-pages/man2/flock.2.html).
- On Windows this is implemented using
  [`LockFileEx`](https://docs.microsoft.com/en-us/windows/win32/api/fileapi/nf-fileapi-lockfileex).

## 🚀 Getting started

First add `fmutex` to your Cargo manifest.

```sh
cargo add fmutex
```

Now use one of the provided functions to lock a file descriptor (Unix) or
handle (Windows) or a file path.

For exclusive locks (only one process can hold the lock):

- [`lock_exclusive()`](#lock_exclusive) to acquire an exclusive lock on a
  file descriptor or handle.
- [`try_lock_exclusive()`](#try_lock_exclusive) to attempt to acquire an
  exclusive lock on a file descriptor or handle.
- [`lock_exclusive_path()`](#lock_exclusive_path) to acquire an exclusive
  lock on a file path.
- [`try_lock_exclusive_path()`](#try_lock_exclusive_path) to attempt to
  acquire an exclusive lock on a file path.

For shared locks (multiple processes can hold the lock simultaneously, but
not when an exclusive lock is held):

- [`lock_shared()`](#lock_shared) to acquire a shared lock on a file
  descriptor or handle.
- [`try_lock_shared()`](#try_lock_shared) to attempt to acquire a shared
  lock on a file descriptor or handle.
- [`lock_shared_path()`](#lock_shared_path) to acquire a shared lock on a
  file path.
- [`try_lock_shared_path()`](#try_lock_shared_path) to attempt to acquire a
  shared lock on a file path.

## 🤸 Usage

### [`lock_exclusive()`][lock_exclusive]

```rust
let fd = fs::OpenOptions::new().create(true).write(true).open(&path)?;

{
    let _guard = fmutex::lock_exclusive(&fd)?;

    // do mutually exclusive stuff here

} // <-- `_guard` dropped here and the lock is released
```

### [`try_lock_exclusive()`][try_lock_exclusive]

```rust
let fd = fs::OpenOptions::new().create(true).write(true).open(&path)?;

match fmutex::try_lock_exclusive(&fd)? {
    Some(_guard) => {

        // do mutually exclusive stuff here

    } // <-- `_guard` dropped here and the lock is released

    None => {
        eprintln!("the lock could not be acquired!");
    }
}
```

### [`lock_exclusive_path()`][lock_exclusive_path]

```rust
let path = "path/to/my/file.txt";

{
    let _guard = fmutex::lock_exclusive_path(path)?;

    // do mutually exclusive stuff here

} // <-- `_guard` dropped here and the lock is released
```

### [`try_lock_exclusive_path()`][try_lock_exclusive_path]

```rust
let path = "path/to/my/file.txt";

match fmutex::try_lock_exclusive_path(path)? {
    Some(_guard) => {

        // do mutually exclusive stuff here

    } // <-- `_guard` dropped here and the lock is released

    None => {
        eprintln!("the lock could not be acquired!");
    }
}
```

### [`lock_shared()`][lock_shared]

```rust
let fd = fs::OpenOptions::new().create(true).write(true).open(&path)?;

{
    let _guard = fmutex::lock_shared(&fd)?;

    // do shared read-only operations here
    // other processes can also acquire shared locks simultaneously

} // <-- `_guard` dropped here and the lock is released
```

### [`try_lock_shared()`][try_lock_shared]

```rust
let fd = fs::OpenOptions::new().create(true).write(true).open(&path)?;

match fmutex::try_lock_shared(&fd)? {
    Some(_guard) => {

        // do shared read-only operations here
        // other processes can also acquire shared locks simultaneously

    } // <-- `_guard` dropped here and the lock is released

    None => {
        eprintln!("the shared lock could not be acquired (file is exclusively locked)!");
    }
}
```

### [`lock_shared_path()`][lock_shared_path]

```rust
let path = "path/to/my/file.txt";

{
    let _guard = fmutex::lock_shared_path(path)?;

    // do shared read-only operations here
    // other processes can also acquire shared locks simultaneously

} // <-- `_guard` dropped here and the lock is released
```

### [`try_lock_shared_path()`][try_lock_shared_path]

```rust
let path = "path/to/my/file.txt";

match fmutex::try_lock_shared_path(path)? {
    Some(_guard) => {

        // do shared read-only operations here
        // other processes can also acquire shared locks simultaneously

    } // <-- `_guard` dropped here and the lock is released

    None => {
        eprintln!("the shared lock could not be acquired (file is exclusively locked)!");
    }
}
```

## License

This project is distributed under the terms of both the MIT license and the Apache License (Version 2.0).

See [LICENSE-APACHE](LICENSE-APACHE) and [LICENSE-MIT](LICENSE-MIT) for details.


[lock_exclusive]: https://docs.rs/fmutex/latest/fmutex/fn.lock_exclusive.html
[lock_exclusive_path]: https://docs.rs/fmutex/latest/fmutex/fn.lock_exclusive_path.html
[lock_shared]: https://docs.rs/fmutex/latest/fmutex/fn.lock_shared.html
[lock_shared_path]: https://docs.rs/fmutex/latest/fmutex/fn.lock_shared_path.html
[try_lock_exclusive]: https://docs.rs/fmutex/latest/fmutex/fn.try_lock_exclusive.html
[try_lock_exclusive_path]: https://docs.rs/fmutex/latest/fmutex/fn.try_lock_exclusive_path.html
[try_lock_shared]: https://docs.rs/fmutex/latest/fmutex/fn.try_lock_shared.html
[try_lock_shared_path]: https://docs.rs/fmutex/latest/fmutex/fn.try_lock_shared_path.html
